# -*- coding: utf-8 -*-
import os
import json
from urllib import request, parse
from .base import BaseTranslator
from settings import DEEPL_API_ENDPOINT


class DeepLTranslator(BaseTranslator):
    """DeepL translator class."""

    def __init__(self, *args, **kwargs):
        """Initialize DeepL translator instance.

        :param **kwargs:
            glossary: Glossary ID to use when translating
        """
        self.glossary = kwargs.get("glossary")
        super().__init__(*args, **kwargs)

    def translate_string(self, text: str) -> str:
        """Translate a specifig string.

        :param text: string to translate
        :return: string translation
        """
        data = {
            "target_lang": self.target_locale,
            "auth_key": os.environ.get("DEEPL_AUTH_KEY"),
            "text": text,
            "preserve_formatting": "1",
        }

        if self.source_locale is not None:
            data["source_lang"] = self.source_locale

        if self.glossary is not None:
            data["glossary_id"] = self.glossary

        data = parse.urlencode(data).encode()

        req = request.Request(DEEPL_API_ENDPOINT, data=data)
        response = request.urlopen(req)  # nosec

        if response.status != 200:
            self.log_translation(
                input_text=text,
                result=f"response status: {response.status}",
                status=self.Status.error,
            )
            return text

        response_data = json.loads(response.read())

        if "translations" not in response_data:
            self.log_translation(
                input_text=text,
                result=f"response empty: {response_data}",
                status=self.Status.error,
            )
            return text

        if len(response_data["translations"]) > 1:
            self.log_translation(
                input_text=text,
                result=f"more than 1 translation: {response_data['translations']})",
                status=self.Status.warning,
            )

        return response_data["translations"][0]["text"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse

from settings import (
    INDENTATION_DEFAULT,
    SLEEP_BETWEEN_API_CALLS,
    ENCODING,
)


def get_parser() -> argparse.ArgumentParser:
    """Construct and return the argument parser for all commands."""
    parser = argparse.ArgumentParser(
        prog="json_translate",
        description="Translate json files using the DeepL API ",
        epilog="Report issues at https://github.com/Saigesp/json-translate/issues",
    )
    parser.add_argument(
        "service",
        help="Translation service to use",
    )
    parser.add_argument(
        "file",
        help="Folder or file to look for translation source",
    )
    parser.add_argument(
        "locale",
        help="Language target to translate",
    )
    parser.add_argument(
        "-sl",
        "--source-locale",
        help="Language translating from (required for glossary)",
    )
    parser.add_argument(
        "-g",
        "--glossary",
        help="ID of glossary to use when translating",
    )
    parser.add_argument(
        "-e",
        "--extend",
        action="store_true",
        help="Extend an existing translation file",
    )
    parser.add_argument(
        "--override",
        action="store_true",
        help="Override existing translation file (don't ask)",
    )
    parser.add_argument(
        "-o",
        "--output",
        help="Output file name",
    )
    parser.add_argument(
        "-i",
        "--indent",
        type=int,
        default=INDENTATION_DEFAULT,
        help="Indentation spaces",
    )
    parser.add_argument(
        "-s",
        "--sleep",
        type=float,
        default=SLEEP_BETWEEN_API_CALLS,
        help="Sleep time between API calls",
    )
    parser.add_argument(
        "--skip",
        nargs="+",
        help="Keys to skip",
    )
    parser.add_argument(
        "--encoding",
        default=ENCODING,
        help="File encoding",
    )
    parser.add_argument(
        "--log",
        action="store_true",
        help="If print translation results",
    )
    parser.add_argument(
        "--formality",
        action="store_true",
        help="Set formality level to FORMAL",
    )
    parser.add_argument(
        "--profanity",
        action="store_true",
        help="Mask profane words and phrases",
    )

    return parser



# -*- coding: utf-8 -*-
import os
from dotenv import load_dotenv

load_dotenv()

DEEPL_API_ENDPOINT = os.getenv(
    "DEEPL_API_ENDPOINT",
    "https://api-free.deepl.com/v2/translate",
)
AWS_REGION_NAME = os.getenv("AWS_REGION_NAME")
AWS_ACCESS_KEY_ID = os.getenv("AWS_ACCESS_KEY_ID")
AWS_SECRET_ACCESS_KEY = os.getenv("AWS_SECRET_ACCESS_KEY")

# Seconds to sleep between API calls
SLEEP_BETWEEN_API_CALLS = float(os.getenv("SLEEP_BETWEEN_API_CALLS", 0.01))

# Default indentation to output the json file
INDENTATION_DEFAULT = int(os.getenv("INDENTATION_DEFAULT", 2))

# Default input file encoding
ENCODING = os.getenv("ENCODING", "utf-8")

# Supported language codes
# fmt: off
DEELP_SUPPORTED_LANGS = ( # https://www.deepl.com/docs-api/translate-text
    "BG", "CS", "DA", "DE", "EL", "EN", "EN-GB", "EN-US", "ES", "ET", "FI", "FR", "HU", "ID",
    "IT", "JA", "KO", "LT", "LV", "NB", "NL", "PL", "PT-BR", "PT-PT", "RO", "RU", "SK",
    "SL", "SV", "TR", "UK", "ZH",
)

AWS_SUPPORTED_LANGS = ( # https://docs.aws.amazon.com/translate/latest/dg/what-is-languages.html
    "af", "sq", "am", "ar", "hy", "az", "bn", "bs", "bg", "ca", "zh", "zh-TW", "hr", "cs",
    "da", "fa-AF", "nl", "en", "et", "fa", "tl", "fi", "fr", "fr-CA", "ka", "de", "el",
    "gu", "ht", "ha", "he", "hi", "hu", "is", "id", "ga", "it", "ja", "kn", "kk", "ko",
    "lv", "lt", "mk", "ms", "ml", "mt", "mr", "mn", "no", "ps", "pl", "pt", "pt-PT", "pa",
    "ro", "ru", "sr", "si", "sk", "sl", "so", "es", "es-MX", "sw", "sv", "ta", "te", "th",
    "tr", "uk", "ur", "uz", "vi", "cy",
)
# fmt: on



# -*- coding: utf-8 -*-
from argparser import get_parser
from languages import get_target_lang_code
from files import (
    get_input_dir_from_file,
    get_file_name_without_extension,
    get_input_file_from_dir,
    get_output_file,
    get_data_to_translate,
    save_results_file,
)
from translators import get_translator


def main():
    """Execute translator command."""
    parser = get_parser()
    args = parser.parse_args()

    input_dir = get_input_dir_from_file(args.file)
    input_file = get_input_file_from_dir(input_dir)
    lang_code = get_target_lang_code(args.service, args.locale)
    json_file_name = get_file_name_without_extension(input_file)

    if lang_code.lower() == json_file_name.lower():
        print("You are trying to translate to the same language!")  # noqa: T201
        exit(1)

    output_file = get_output_file(
        output=args.output,
        lang_code=lang_code,
        input_file=input_file,
        extend=args.extend,
        override=args.override,
    )
    data_to_translate = get_data_to_translate(
        input_file=input_file,
        output_file=output_file,
        extend=args.extend,
        encoding=args.encoding,
    )
    translator = get_translator(args.service)(
        target_locale=lang_code.upper(),
        source_locale=args.source_locale,
        sleep=args.sleep,
        skip=args.skip,
        encoding=args.encoding,
        log_translations=args.log,
        glossary=args.glossary,
        formality=args.formality,
        profanity=args.profanity,
    )
    results = translator.translate(
        data=data_to_translate,
    )
    save_results_file(
        data=results,
        output_file=output_file,
        extend=args.extend,
        indent=args.indent,
        encoding=args.encoding,
    )



# -*- coding: utf-8 -*-
from settings import DEELP_SUPPORTED_LANGS, AWS_SUPPORTED_LANGS


def get_target_lang_code(service: str, locale: str) -> str:
    """Get language code from input.

    :param service: translation service to use
    :param locale: locate target to use
    :return: output locale code
    """
    lang_code = locale or ""

    while len(lang_code) > 5:
        lang_code = input(
            "Language code to translate to (usually 2 letters, unless a supported region like EN-GB): "
        )

    if service == "deepl" and lang_code.upper() not in DEELP_SUPPORTED_LANGS:
        # TODO: Log properly
        print(  # noqa: T201
            f"Language {lang_code} is not supported by DeepL.",
            "Check the supported ones in https://www.deepl.com/docs-api/translate-text",
        )
        exit(1)

    if service == "aws" and lang_code.lower() not in AWS_SUPPORTED_LANGS:
        # TODO: Log properly
        print(  # noqa: T201
            f"Language {lang_code} is not supported by AWS.",
            "Check the supported ones in https://docs.aws.amazon.com/translate/latest/dg/what-is-languages.html",
        )
        exit(1)

    return lang_code



# -*- coding: utf-8 -*-
from .deepl import DeepLTranslator
from .aws import AWSTranslator


def get_translator(service: str):
    """Get translator service class."""
    if service == "aws":
        return AWSTranslator

    return DeepLTranslator



# -*- coding: utf-8 -*-
import os
import sys

sys.path.append(os.path.dirname(os.path.realpath(__file__)))  # noqa: PTH120

import files  # noqa: E402
import languages  # noqa: E402
import argparser  # noqa: E402
import settings  # noqa: E402



# -*- coding: utf-8 -*-
import time
from abc import ABC, abstractmethod
from settings import ENCODING, SLEEP_BETWEEN_API_CALLS


class BaseTranslator(ABC):
    """Translator Abstract base class.

    Use this class to implement translators
    """

    cached = {}

    class Status:
        """Translation result status."""

        success: int = 0
        warning: int = 1
        error: int = 2

    def __init__(
        self,
        target_locale: str,
        *,
        source_locale: str = None,
        skip: list = None,
        sleep: float = SLEEP_BETWEEN_API_CALLS,
        encoding: str = ENCODING,
        log_translations: bool = False,
        **kwargs,
    ):
        """Initialize base translator instance.

        :param target_locale: locale to translate
        :param skip: list of keys to ignore
        :param sleep: sleep time between API calls
        :param encoding: encoding (utf-8, latin-1 etc)
        :param log_translations: if print translation results
        """
        self.skip_keys = skip or []
        self.target_locale = target_locale
        self.source_locale = source_locale
        self.sleep = sleep
        self.encoding = encoding
        self.log_translations = log_translations

    def translate(self, data: dict) -> dict:
        """Translate nested data.

        :param data: data to translate
        :return: translation
        """
        return self._iterate_over_keys(data)

    def _iterate_over_keys(self, data) -> list | dict | str:
        """Iterate on data and translate the corresponding values.

        :param data: data to iterate
        :return: translated block
        """
        if isinstance(data, dict):
            # Value is hierarchical, so iterate it
            return self._get_dict_iteration(data)

        if isinstance(data, list):
            # Value is multiple, so iterate it
            return [self._iterate_over_keys(value) for value in data]

        if isinstance(data, str):
            # Value is string, so translate it
            return self._get_string_iteration(data)

        if isinstance(data, (bool | int | float)):
            # Value is boolean or numerical, return same value
            return data

        raise Exception(f"Can't determine {data} type")

    def _get_dict_iteration(self, data: dict) -> dict:
        result = {}
        for key, value in data.items():
            if key in self.skip_keys:
                result[key] = value
            else:
                result[key] = self._iterate_over_keys(value)
        return result

    def _get_string_iteration(self, text: str) -> str:
        if not isinstance(text, str) or text == "":
            return text

        cached_result = self.cached.get(text)
        if cached_result:
            self.log_translation(text, f"{cached_result} (cached)")
            return cached_result

        time.sleep(self.sleep)

        result = self.translate_string(text)
        self.log_translation(text, result)

        decoded = self.decode(result)
        self.cached[text] = decoded

        return decoded

    def decode(self, text: str) -> str:
        """Decode text."""
        return str(text)  # TODO: improve decoding

    def log_translation(
        self,
        input_text: str,
        result: str = "",
        status: int = 0,
    ) -> None:
        """Log translations result."""
        if not self.log_translations:
            return

        if status != self.Status.success:
            result = f" ({result})"
            translation = input_text

        if status == self.Status.warning:
            str_status = "warning: "
        elif status == self.Status.error:
            str_status = "error: "
        else:
            str_status = ""
            translation = f"{input_text} -> {result}"
            result = ""

        # TODO: log properly
        print(f"{str_status}{translation}{result}")  # noqa: T201

    @abstractmethod
    def translate_string(self):
        """Require translate_string to be implemented."""
        raise NotImplementedError

    def __repr__(self):
        """Repr the child classes."""
        return f"{self.__class__.__name__}({self.target_locale.lower()})"



# -*- coding: utf-8 -*-
import sys
import setuptools
from pathlib import Path

if sys.version_info.major == 2:
    sys.exit("Python 2 is not supported anymore. The last supported version is 3.10.0")

BASE_DIR = Path(__file__).resolve().parent
version = "v0.4.4"

with Path.open(BASE_DIR / "README.md") as fh:
    long_description = fh.read()


def get_reqs(filename):
    """Read file per-line."""
    with Path.open(BASE_DIR / filename) as reqs_file:
        return reqs_file.readlines()


reqs = get_reqs("requirements.txt")

setuptools.setup(
    name="json_translate",
    version=version,
    author="Saigesp",
    author_email="saigesp@gmail.com",
    description="CLI tool to translate json files using different external services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires=">=3.10",
    install_requires=reqs,
    classifiers=[
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
    ],
    entry_points={
        "console_scripts": [
            "json_translate=json_translate.commands:main",
        ],
    },
)



# -*- coding: utf-8 -*-
class DataDiff:
    """DataDiff class."""

    def __init__(self, initial: dict, minus: dict):
        """Initialize DataDiff object."""
        self.initial = initial
        self.minus = minus
        self.diff = self._get_diff(self.initial, self.minus)

    def to_dict(self) -> dict:
        """Convert data difference to dict."""
        return self.diff

    def _get_diff(self, initial, minus) -> dict | list:
        if isinstance(initial, dict):
            return self._get_dict_diff(initial, minus)
        if isinstance(initial, list):
            return self._get_list_diff(initial, minus)

        raise Exception("Only dict and list supported")

    def _get_dict_diff(self, initial: dict, minus: dict) -> dict:
        keys1 = initial.keys()
        items2 = minus.items()
        missing_keys = {i[0]: i[1] for i in items2 if i[0] not in keys1}

        for key, val in initial.items():
            if not isinstance(val, dict | list):
                continue
            diff = self._get_diff(initial.get(key), minus.get(key))
            if not diff:
                continue
            missing_keys[key] = diff

        return missing_keys

    def _get_list_diff(self, initial: list, minus: list) -> list:
        return minus



# -*- coding: utf-8 -*-
import os
import re
import json
from pathlib import Path
from mergedeep import merge
from datadiff import DataDiff


def get_input_dir_from_file(file: os.PathLike) -> os.PathLike:
    """Get file folder."""
    return os.path.normpath(file)


def get_file_name_without_extension(input_file: os.PathLike) -> str:
    """Get file name without extension."""
    names = Path(input_file).name.split(".")
    return ".".join(names[:-1])


def find_files(path: str) -> list:
    """Find all json files in folder.

    :param path: directory path to list files
    :return: files in directory
    """
    return [file for file in os.listdir(path) if re.search(r"\.json$", file)]


def get_input_file(json_files: list, input_dir: os.PathLike) -> str:
    """Get input file from folder.

    :param json_files: list of files in directory
    :param input_dir: directory containing these files
    :return: file to translate
    """
    if len(json_files) == 0:
        print("No files found")  # noqa: T201
        exit(1)

    if len(json_files) == 1:
        return json_files[0]

    print("Choose the file to use as source file:")  # noqa: T201
    for idx, file in enumerate(json_files):
        print(f"[{idx}] {file}")  # noqa: T201

    file_idx = input("Type file number: ")
    return Path(input_dir) / json_files[int(file_idx)]


def get_input_file_from_dir(input_dir: str) -> os.PathLike:
    """Get input file from dir.

    :param input_dir: selected folder to search for files
    :return: file to translate
    """
    input_dir_norm = os.path.normpath(input_dir)

    if Path(input_dir_norm).is_dir():
        json_files = find_files(input_dir_norm)
        return get_input_file(json_files, input_dir_norm)

    if not input_dir_norm.endswith(".json"):
        print(  # noqa: T201
            "You must select a json file or a folder containing json files"
        )
        exit(1)

    if not Path(input_dir_norm).is_file():
        print("File not found:", input_dir_norm)  # noqa: T201
        exit(1)

    return Path(input_dir_norm)


def get_output_file(
    *,
    output: str,
    lang_code: str,
    input_file: str,
    extend: bool = False,
    override: bool = False,
) -> str:
    """Get output file.

    :param output: output file name
    :param lang_code: output file language code
    :param input_file: input file
    :param extend: if output file must be extended
    :param override: if output file must be overrided
    :return: file to output translations
    """
    output_file_name = output if output else f"{lang_code.lower()}.json"

    if not output_file_name.endswith(".json"):
        output_file_name += ".json"

    output_file = Path(input_file).parent / output_file_name

    if output_file.exists() and not extend:
        if override:
            return output_file
        ask_override = input(
            f"File {output_file_name} already exists."
            " Do you want to override it? [Y/n] "
        )
        if ask_override.lower() not in ("y", "yes", "ok", ""):
            output_file_name = input("Enter the new file name: ")
            if not output_file_name.endswith(".json"):
                output_file_name += ".json"

            return Path(input_file).parent / output_file_name

    return output_file


def get_data_to_translate(
    input_file: os.PathLike,
    *,
    output_file: os.PathLike = None,
    extend: bool = False,
    encoding: str = "utf8",
) -> dict:
    """Get data to translate.

    :param input_file: file to translate
    :param output_file: file to save
    :param extend: if output file must be extended
    :param encoding: file encoding
    """
    if extend and (output_file is None or not output_file.exists()):
        print("Existing file to extend not found")  # noqa: T201
        exit(1)

    with Path.open(input_file, "r", encoding=encoding) as file:
        input_data = json.load(file)

    if not extend:
        return input_data

    with Path.open(output_file, "r", encoding=encoding) as file:
        existing_data = json.load(file)

    diff = DataDiff(existing_data, input_data)
    return diff.to_dict()


def save_results_file(
    data: dict,
    output_file: os.PathLike,
    *,
    extend: bool = False,
    indent: int = 2,
    encoding: str = "utf8",
) -> None:
    """Write output file.

    :param data: dict object to dump into file
    :param output_file: output file path
    :param extend: if output file must be extended
    :param indent: json indentation
    :param encoding: file encoding
    """
    if extend:
        if output_file is None or not output_file.exists():
            raise Exception("Existing file to extend not found")

        with Path.open(output_file, "r", encoding=encoding) as file:
            data = merge(json.load(file), data)

    with Path.open(output_file, "w", encoding=encoding) as file:
        json.dump(data, file, indent=indent, ensure_ascii=False)

    print(f"Results saved on {output_file}")  # noqa: T201



# -*- coding: utf-8 -*-
import boto3
from .base import BaseTranslator
from settings import (
    AWS_REGION_NAME,
    AWS_ACCESS_KEY_ID,
    AWS_SECRET_ACCESS_KEY,
)


class AWSTranslator(BaseTranslator):
    """AWS translator class."""

    def __init__(self, *args, **kwargs):
        """Initialize AWS translator instance.

        :param **kwargs:
            formality: level of formality for translations
            profanity: mask profane words and phrases
        """
        self.client = boto3.client(
            "translate",
            region_name=AWS_REGION_NAME,
            aws_access_key_id=AWS_ACCESS_KEY_ID,
            aws_secret_access_key=AWS_SECRET_ACCESS_KEY,
        )
        self.formality = "FORMAL" if kwargs.get("formality") else "INFORMAL"
        self.profanity = "MASK" if kwargs.get("profanity") else None

        if kwargs.get("source_locale") is None:
            raise Exception("Param 'source_locale' is required in AWSTranslator")

        super().__init__(*args, **kwargs)

    def translate_string(self, text: str) -> str:
        """Translate a specifig string.

        :param text: string to translate
        :return: string translation
        """
        settings = {}

        if self.formality is not None:
            settings["Formality"] = self.formality.upper()

        if self.profanity is not None:
            settings["Profanity"] = self.profanity.upper()

        response = self.client.translate_text(
            Text=text,
            SourceLanguageCode=self.source_locale,
            TargetLanguageCode=self.target_locale,
            Settings=settings,
        )

        meta = response.get("ResponseMetadata")

        if meta.get("HTTPStatusCode") != 200:
            self.log_translation(
                input_text=text,
                result=f"response status: {meta.get('HTTPStatusCode')}",
                status=self.Status.error,
            )
            return text

        if not response.get("TranslatedText"):
            self.log_translation(
                input_text=text,
                result=f"response empty: {response}",
                status=self.Status.error,
            )
            return text

        return response.get("TranslatedText")
